
/*
*   spork.c
*
*   This is a very simple program that reads Hot Little Therms on Windows
*   machines.  I wrote it just for testing the device, but it may be useful 
*   as an programming example.
*
*   01 Sep 97 Manchek
*/

#include <Windows.h>
#include <stdio.h>
#include <ctype.h>

#define LENGTHOF(x) (sizeof(x)/sizeof((x)[0]))


/*  hdump()
*
*   Hex dump of byte array.
*/

void
hdump(
    FILE *ff,       /* output file */
    char *p,        /* bytes */
    int n)          /* length */
{
    int i;
    for (i = 0; i < n; i++)
	fprintf(ff, "%02x%s", 0xff & (int)p[i],
	    (i < n - 1) ? (((i & 15) == 15) ? "\n" : " ") : "\n");
}


/*  getinput()
*
*   like fgets() but strips off any trailing \n.
*/

int
getinput(
    char *buf,
    int len,
    FILE *ff )
{
    int i;

    if (!fgets(buf, len - 1, ff)) {
	buf[0] = 0;
	return 0;
    }
    buf[len - 1] = 0;
    i = strlen(buf);
    if (i > 0 && buf[i - 1] == '\n') {
	buf[i - 1] = 0;
    }
    return 1;
}


/*  PostError()
*
*   error "handler".
*/

void
PostError(
   char *message )
{
    LPVOID lpMsgBuf;
 
    FormatMessage( 
	FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
	NULL,
	GetLastError(),
	MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
	(LPTSTR) &lpMsgBuf,
	0,
	NULL);

/*
    MessageBox( NULL, lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );
*/
    fprintf(stderr, "PostError: %s: %s\n", message, lpMsgBuf);

    LocalFree( lpMsgBuf );

    exit(1);
}


/*  openPort()
*
*   open a serial port and configure it for use with HLT.
*   returns handle to the port if successful.
*/

HANDLE
openPort(
   char *name )
{
    HANDLE tty;
    DCB dcb;
    COMMTIMEOUTS cto;

    tty = CreateFile(name,
	GENERIC_READ|GENERIC_WRITE,
	0,
	NULL,
	OPEN_EXISTING,
	0,
	NULL);
/*
    printf("0x%x\n", tty);
*/
    if (tty == INVALID_HANDLE_VALUE) {
	PostError("can't open tty");
    }
    if (GetCommState(tty, &dcb) != TRUE) {
	PostError("can't GetCommState");
    }
    if (BuildCommDCB("baud=1200 parity=N data=8 stop=1", &dcb) != TRUE) {
	PostError("can't BuildCommDCB");
    }
    if (SetCommState(tty, &dcb) != TRUE) {
	PostError("can't SetCommState");
    }
    cto.ReadIntervalTimeout = 0;
    cto.ReadTotalTimeoutMultiplier = 0;
    cto.ReadTotalTimeoutConstant = 1000;
    cto.WriteTotalTimeoutMultiplier = 0;
    cto.WriteTotalTimeoutConstant = 0;
    if (SetCommTimeouts(tty, &cto) != TRUE) {
	PostError("can't SetCommTimeouts");
    }

    return tty;
}


/*  readTherm()
*
*   send address command to HLT and read reply.  reply must be exactly
*   8 characters ("[+-]ddd.d\r\n").  times out in 1 second in case no
*   therm is connected to the serial port.
*
*   on return, reply[] contains either the therm response, stripped of
*   the \r and \n, or "timeout" if nothing was received.  reply must
*   point to a buffer large enough to hold the reply.
*/

void
readTherm(
    HANDLE tty,
    int ad,
    char *reply )
{
    int nn;
    char buf[100];

    buf[0] = ad;
    if (WriteFile(tty, buf, 1, &nn, NULL) != TRUE) {
	PostError("can't WriteFile");
    }
/*
    fprintf(stderr, "wrote %d byte\n", nn);
*/

    nn = 0;
    if (ReadFile(tty, buf, 8, &nn, NULL) != TRUE) {
	PostError("can't ReadFile");
    }
    if (nn == 8) {
	strncpy(reply, buf, 6);
	reply[6] = 0;
    } else {
	strcpy(reply, "timeout");
    }
#if 0
    if (ReadFileEx(tty, buf, 8, &nn, NULL) != TRUE) {
	PostError("can't ReadFile");
    }
    if (SleepEx(1000, TRUE) == 0) {
	strcpy(reply, "timeout");
    } else {
	strncpy(reply, buf, 6);
	reply[6] = 0;
    }
#endif
/*
    fprintf(stderr, "read %d byte\n", nn);
*/
}

/*  xtoi()
*
*   Yet another version of ascii hex to integer
*/

int
xtoi(
    char *p)
{
    int i = 0;
    char c;

    if (p[0] == '0' && (p[1] == 'X' || p[1] == 'x'))
	p += 2;
    while (isxdigit(c = *p++)) {
	i = (i << 4) + c - (isdigit(c) ? '0' : (isupper(c) ? 'A' : 'a') - 10);
    }
    return i;
}


int
axtoi(
    char *p )
{
    if (p[0] == '0' && (p[1] == 'x' || p[1] == 'X'))
	return xtoi(p);
    return atoi(p);
}


int
main(
    int argc,
    char **argv )
{
    HANDLE tty;
    int d;
    int nprobes;
    char c, *p, *q;
    int n;
    char ttname[200];
    char buf[200];
    int probes[50];

    /*
    * get tty name and probe addresses
    */

    fputs("tty: ", stdout);
    fflush(stdout);
    getinput(ttname, sizeof(ttname)-1, stdin);

    fputs("probe addresses (decimal or 0x...): ", stdout);
    fflush(stdout);
    getinput(buf, sizeof(buf)-1, stdin);

    /*
    * convert list of addresses
    */

    nprobes = 0;
    p = buf;
    while (*p) {
	while (*p && !isxdigit(*p))
	    *p++;
	for (q = p; isxdigit(*q) || *q == 'x' || *q == 'X'; q++) ;
	if (*p) {
	    c = *q;
	    *q = 0;
	    probes[nprobes] = axtoi(p);
	    nprobes++;
	    *q = c;
	    p = q;
	}
    }
    
/*
    strcpy(ttname, "\\\\.\\com4");
*/
    tty = openPort(ttname);
    n = 0;
    while (1) {
	for (d = 0; d < nprobes; d++) {
	    readTherm(tty, probes[d], buf);
	    n++;
	    printf("0x%x = <%s>\n", probes[d], buf);
	}
    }

    exit(0);
}

